#include <IRremote.h>                    //include IR remote libraries

int RECV_PIN = 12;                       //pin to read the signal from the infrared module
IRrecv irrecv(RECV_PIN);
decode_results results;

//variables
unsigned long CurrentValue = 0;          //value received from the infrared module
unsigned long StoredCode = 0;            //saved code value
unsigned long previousSwitchTime = 0;    //last time marker that the output was toggled
unsigned long switchTimeDifference = 0;  //how long since the last output toggle
const int buttonPin = 10;                // the number of the pushbutton pin
const int programLedPin =  8;            // the number of the program LED pin
const int indicatorLEDPin =  7;          // the number of the output LED pin
const int relayPin = 4;

int buttonState = 0;                     //variable for reading the pushbutton status
int RecordState = 0;                     //is the reciever in record mode
int outputState = 0;                     //is the output on or off



void setup()
{
  Serial.begin(9600);
  irrecv.enableIRIn();                   //Start the receiver

  pinMode(buttonPin, INPUT);             //set digital pin modes
  pinMode(programLedPin, OUTPUT);    
  pinMode(indicatorLEDPin, OUTPUT);      
  pinMode(relayPin, OUTPUT);  
}


void loop() 
{
 if (irrecv.decode(&results))                                    //if a signal is detected
 {
  CurrentValue = (results.value);                                //save value
  Serial.println(CurrentValue);                                  //displays stored code for reference
 
  switchTimeDifference = millis() - previousSwitchTime;          //calculate time since last switch
  
  if(CurrentValue == StoredCode && switchTimeDifference > 1000)  // if the recieved value equals the programed value and the last state switch was more than 1 second ago, then toggle the output state
   {
    outputState = !outputState;
    previousSwitchTime = millis();
   }
 
  if (RecordState == 1)                                          // if the record mode is activated store the current value as the programed value
   {
    StoredCode = CurrentValue;
    RecordState = 0;
    digitalWrite(programLedPin, LOW);  
    Serial.println(StoredCode);                                 //displays stored code for reference
   }
     
 
  irrecv.resume();                                              // Receive the next value
 }
 else                                                           //if no signal is detected, then the current value is 0
 {
  CurrentValue = 0;
 }
  
 buttonState = digitalRead(buttonPin);                          // check if the record button is pressed.
 
 if (buttonState == HIGH)                                       // if it is, the buttonState is HIGH:
  {   
   while (buttonState == HIGH)                                  //wait for the button to be released
   {
    buttonState = digitalRead(buttonPin);
   }
 
   digitalWrite(programLedPin, HIGH);                          //turn on the LED to indicate that record mode is on
   RecordState = 1;
  }
 
 if(outputState == 1)                                          //set the appropriate output state
 {
  digitalWrite(indicatorLEDPin, HIGH);
  digitalWrite(relayPin, HIGH);
 }
 else 
 {
  digitalWrite(indicatorLEDPin, LOW);
  digitalWrite(relayPin, LOW);
 }
 
}
